@@ -4,6 +4,7 @@ import gobject
 import time
 log = logging.getLogger("Vfs")
 
+import urllib
 import conduit
 import conduit.utils.Singleton as Singleton
 
@@ -19,6 +20,20 @@ else:
 VolumeMonitor   = FileImpl.VolumeMonitor
 FileMonitor     = FileImpl.FileMonitor     
 FolderScanner   = FileImpl.FolderScanner
+FileTransfer    = FileImpl.FileTransferImpl
+
+def backend_supports_remote_uri_schemes():
+    """
+    @returns: True if the file implementation supports non-local (file://)
+    uri schemes
+    """
+    return len(FileImpl.FileImpl.SCHEMES) > 1 and "file://" in FileImpl.FileImpl.SCHEMES
+
+def backend_name():
+    """
+    @returns: The name of the selected file impl backend
+    """
+    return FileImpl.FileImpl.NAME
 
 def uri_is_valid(uri):
     """
@@ -32,31 +47,19 @@ def uri_join(first, *rest):
     Joins multiple uri components. Performs safely if the first
     argument contains a uri scheme
     """
-    first = conduit.utils.ensure_string(first)
-    return os.path.join(first,*rest)
+    return FileImpl.FileImpl.uri_join(first,*rest)
 
 def uri_get_scheme(uri):
     """
     @returns: The scheme (file,smb,ftp) for the uri, or None on error
     """
-    try:
-        scheme,path = uri.split("://")
-        return scheme
-    except exceptions.ValueError:
-        return None
+    return FileImpl.FileImpl.uri_get_scheme(uri)
     
 def uri_get_relative(fromURI, toURI):
     """
     Returns the relative path fromURI --> toURI
     """
-    fromURI = conduit.utils.ensure_string(fromURI)
-    toURI = conduit.utils.ensure_string(toURI)
-    rel = toURI.replace(fromURI,"")
-    #strip leading /
-    if rel[0] == os.sep:
-        return rel[1:]
-    else:
-        return rel
+    return FileImpl.FileImpl.uri_get_relative(fromURI, toURI)
     
 def uri_open(uri):
     """
@@ -151,19 +154,24 @@ def uri_sanitize_for_filesystem(uri, filesystem=None):
     Also see:    
     http://bugzilla.gnome.org/show_bug.cgi?id=309584#c20
     """
+    
     uri = conduit.utils.ensure_string(uri)
     import string
 
     ILLEGAL_CHARS = {
-        "fat"       :   "\\:*?\"<>|/",
-        "vfat"      :   "\\:*?\"<>|/",
-        "msdos"     :   "\\:*?\"<>|/",
-        "msdosfs"   :   "\\:*?\"<>|/",
-        "ntfs"      :   "\\:*?\"<>|/"
+        "fat"       :   "\\:*?\"<>|",
+        "vfat"      :   "\\:*?\"<>|",
+        "msdos"     :   "\\:*?\"<>|",
+        "msdosfs"   :   "\\:*?\"<>|",
+        "ntfs"      :   "\\:*?\"<>|"
     }
 
     illegal = ILLEGAL_CHARS.get(filesystem,None)
     if illegal:
+        
+        #call urllib.unescape otherwise for example ? is rapresented as %3F
+        uri = urllib.unquote(uri)
+
         #dont escape the scheme part
         idx = uri.rfind("://")
         if idx == -1:
@@ -171,12 +179,14 @@ def uri_sanitize_for_filesystem(uri, filesystem=None):
         else:
             start = idx + 3        
 
-        #replace illegal chars with a space, ignoring the scheme
-        return uri[0:start]+uri[start:].translate(string.maketrans(
-                                                illegal,
-                                                " "*len(illegal)
-                                                )
-                                            )
+        #replace illegal chars with a -, ignoring the scheme (don't use a space because you can't create a directory with just a space)
+        ret = uri[0:start]+uri[start:].translate(string.maketrans(
+                illegal,
+                "_"*len(illegal)
+                )
+                                                 )
+        ret = uri[0:start]+urllib.quote (ret[start:])
+        return ret
     return uri
     
 def uri_is_folder(uri):
