@@ -7,7 +7,7 @@ import logging
 log = logging.getLogger("modules.Backpack")
 
 import conduit
-import conduit.Utils as Utils
+import conduit.utils as Utils
 import conduit.dataproviders.DataProvider as DataProvider
 import conduit.Exceptions as Exceptions
 from conduit.datatypes import Rid
@@ -19,12 +19,16 @@ import backpack
 MODULES = {
 	"BackpackNoteSink" : { "type": "dataprovider" }
 }
+log.info("Module Information: %s" % Utils.get_module_information(backpack, None))
 
 class BackpackBase(DataProvider.DataProviderBase):
+    _configurable_ = True
     def __init__(self, *args):
         DataProvider.DataProviderBase.__init__(self)
-        self.username = ""
-        self.apikey = ""
+        self.update_configuration(
+            username = "",
+            apikey = ""
+        )
         self.ba = None
         self.loggedIn = False
 
@@ -62,8 +66,9 @@ class BackpackNoteSink(DataProvider.DataSink, BackpackBase):
     def __init__(self, *args):
         DataProvider.DataSink.__init__(self)
         BackpackBase.__init__(self, *args)
-
-        self.storeInPage = "Conduit"
+        self.update_configuration(
+            storeInPage = 'Conduit',
+        )
         self.pageID = None
         #there is no way to pragmatically see if a note exists so we list them
         #and cache the results. 
@@ -96,31 +101,18 @@ class BackpackNoteSink(DataProvider.DataSink, BackpackBase):
             self._notes[title] = (uid,timestamp,text)
             log.debug("Found existing note: %s (uid:%s timestamp:%s)" % (title, uid, timestamp))
 
-    def configure(self, window):
-        tree = Utils.dataprovider_glade_get_widget(
-                        __file__, 
-                        "config.glade",
-                        "BackpackNotesSinkConfigDialog")
-        
-        #get a whole bunch of widgets
-        usernameEntry = tree.get_widget("username")
-        apikeyEntry = tree.get_widget("apikey")
-        pagenameEntry = tree.get_widget("pagename")        
-        
-        #preload the widgets
-        usernameEntry.set_text(self.username)
-        apikeyEntry.set_text(self.apikey)
-        pagenameEntry.set_text(self.storeInPage)        
-        
-        dlg = tree.get_widget("BackpackNotesSinkConfigDialog")
-
-        response = Utils.run_dialog(dlg, window)
-        if response == True:
-            self.username = usernameEntry.get_text()
-            self.storeInPage = pagenameEntry.get_text()
-            if apikeyEntry.get_text() != self.apikey:
-                self.apikey = apikeyEntry.get_text()
-        dlg.destroy()
+    def config_setup(self, config):
+        config.add_section("Account details")
+        config.add_item("Login", "text",
+            config_name = "username"
+        )
+        config.add_item("API key", "text", 
+            config_name = "apikey"
+        )
+        config.add_section("Saved notes")
+        config.add_item("Save notes in page", "text",
+            config_name = "storeInPage"
+        )            
 
     def get(self, LUID):
         for title in self._notes:
@@ -170,13 +162,3 @@ class BackpackNoteSink(DataProvider.DataSink, BackpackBase):
 
     def get_UID(self):
         return "%s:%s" % (self.username,self.storeInPage)
-
-    def get_configuration(self):
-        return {
-            "username" : self.username,
-            "apikey" : self.apikey,
-            "storeInPage" : self.storeInPage
-            }
-
-
-
